@import url("stylesheet.css");

#panel.bottom-panel {
    border-top-width: 1px;
    border-bottom-width: 0px;
    height: 2.25em ;
  }

  .bottom-panel .window-button > StWidget {
    background-gradient-drection: vertical;
    background-color: #fff;
    background-gradient-start: #fff;
    background-gradient-end: #eee;
    color: #000;
    -st-natural-width: 18.7em;
    max-width: 18.75em;
    color: #2e3436;
    background-color: #eee;
    border-radius: 2px;
    padding: 3px 6px 1px;
    box-shadow: inset -1px -1px 1px rgba(0,0,0,0.5);
    text-shadow: 0 0 transparent;
  }

  .bottom-panel .window-button:hover > StWidget {
    background-color: #f9f9f9;
  }

  .bottom-panel .window-button:active > StWidget,
  .bottom-panel .window-button:focus > StWidget {
    box-shadow: inset 1px 1px 2px rgba(0,0,0,0.5);
  }

  .bottom-panel .window-button.focused > StWidget {
    background-color: #ddd;
    box-shadow: inset 1px 1px 1px rgba(0,0,0,0.5);
  }

  .bottom-panel .window-button.focused:hover > StWidget {
    background-color: #e9e9e9;
  }

  .bottom-panel .window-button.minimized > StWidget {
    color: #888;
    box-shadow: inset -1px -1px 1px rgba(0,0,0,0.5);
  }
